#include "board.h"
#include <sstream>
using namespace std;
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

class BoardTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(BoardTest);
  CPPUNIT_TEST(testNStones);
  CPPUNIT_TEST(testCount);
  CPPUNIT_TEST(testCanWinByDrop);
  CPPUNIT_TEST(testCanWinByMove);
  CPPUNIT_TEST(testNextDrops);
  CPPUNIT_TEST(testNextMoves);
  CPPUNIT_TEST_SUITE_END();
 public:
  void testNStones();
  void testCount();
  void testCanWinByDrop();
  void testCanWinByMove();
  void testNextDrops();
  void testNextMoves();
};


CPPUNIT_TEST_SUITE_REGISTRATION(BoardTest);

void BoardTest::testNStones()
{
  {
    Board b(". . . .\n"
	    " . . .\n"
	    ". + + -\n"
	    " . . .\n"
	    ". - . .\n"
	    " . . .\n"
	    ". . . .\n");
    CPPUNIT_ASSERT_EQUAL(4,b.nStones());
  }
  {
    Board b(". . . .\n"
	    " . . .\n"
	    ". . + -\n"
	    " . . .\n"
	    ". - . .\n"
	    " . . .\n"
	    ". . . .\n");
    CPPUNIT_ASSERT_EQUAL(3,b.nStones());
  }
}

void BoardTest::testCount()
{
  {
    Board b(". + . .\n"
	    " . . .\n"
	    ". + + -\n"
	    " . . .\n"
	    ". - . .\n"
	    " . - .\n"
	    ". . . .\n");
    int i=pos2IndexTable.getIndex(4);
    CPPUNIT_ASSERT_EQUAL(2,b.count(i,Board::RIGHT));
    CPPUNIT_ASSERT_EQUAL(1,b.count(i,Board::RIGHT+Board::UP));
    CPPUNIT_ASSERT_EQUAL(0,b.count(i,Board::UP));
    CPPUNIT_ASSERT_EQUAL(0,b.count(i,Board::UP+Board::LEFT));
    CPPUNIT_ASSERT_EQUAL(0,b.count(i,Board::UP));
    CPPUNIT_ASSERT_EQUAL(0,b.count(i,Board::LEFT));
    CPPUNIT_ASSERT_EQUAL(0,b.count(i,Board::DOWN+Board::LEFT));
    CPPUNIT_ASSERT_EQUAL(0,b.count(i,Board::DOWN));
    CPPUNIT_ASSERT_EQUAL(0,b.count(i,Board::DOWN+Board::RIGHT));
  }
}
void BoardTest::testCanWinByDrop()
{
  {
    Board b(". + . -\n"
	    " . . .\n"
	    ". + + -\n"
	    " . . .\n"
	    ". - . .\n"
	    " . - .\n"
	    ". . . .\n");
    CPPUNIT_ASSERT(b.canWinByDrop());
  }
  {
    Board b(". . . -\n"
	    " . . .\n"
	    "+ + - .\n"
	    " . . .\n"
	    ". . . .\n"
	    " . - .\n"
	    ". . . .\n");
    CPPUNIT_ASSERT(!b.canWinByDrop());
  }
  {
    Board b(". . - -\n"
	    " . . .\n"
	    "+ + + .\n"
	    " . . .\n"
	    ". - . .\n"
	    " . - .\n"
	    ". . . .\n");
    CPPUNIT_ASSERT(!b.canWinByDrop());
  }
}

void BoardTest::testCanWinByMove()
{
  {
    Board b(". + . -\n"
	    " . . .\n"
	    ". + + -\n"
	    " + . .\n"
	    ". - - .\n"
	    " . . .\n"
	    ". . . .\n");
    CPPUNIT_ASSERT(b.canWinByMove());
  }
  {
    Board b(". . + -\n"
	    " . . .\n"
	    "+ + - .\n"
	    " . . .\n"
	    ". + . -\n"
	    " . - .\n"
	    ". . . .\n");
    CPPUNIT_ASSERT(!b.canWinByMove());
  }
  {
    Board b(". . - -\n"
	    " . + .\n"
	    "+ + . +\n"
	    " . . .\n"
	    ". - . .\n"
	    " . - .\n"
	    ". . . .\n");
    CPPUNIT_ASSERT(!b.canWinByMove());
  }
}

bool isMember(Board const& b,vector<PackedBoard> const& nexts)
{
  PackedBoard packed=b.packRev();
  for(size_t i=0;i<nexts.size();i++)
    if(packed==nexts[i]) return true;
  return false;
}

void BoardTest::testNextDrops()
{
  {
    Board b(". + . -\n"
	    " . . .\n"
	    ". + . -\n"
	    " . . .\n"
	    ". - . .\n"
	    " . . .\n"
	    ". . - +\n");
    vector<PackedBoard> nexts;
    b.nextDrops(nexts);
    {
      Board b1(". + . -\n"
	       " + . .\n"
	       ". + . -\n"
	       " . . .\n"
	       ". - . .\n"
	       " . . .\n"
	       ". . - +\n");
      CPPUNIT_ASSERT(isMember(b1,nexts));
    }
    {
      Board b1(". + . -\n"
	       " . . .\n"
	       ". + . -\n"
	       " . . .\n"
	       ". - . .\n"
	       " . . .\n"
	       ". + - +\n");
      CPPUNIT_ASSERT(!isMember(b1,nexts));
    }
    {
      Board b1(". + . -\n"
	       " . . -\n"
	       ". + . -\n"
	       " . . .\n"
	       ". . . -\n"
	       " . . .\n"
	       ". + . +\n");
      CPPUNIT_ASSERT(isMember(b1,nexts));
    }
    {
      Board b1(". + . -\n"
	       " . . .\n"
	       ". + . -\n"
	       " . . .\n"
	       ". . . -\n"
	       " . . .\n"
	       ". + - +\n");
      CPPUNIT_ASSERT(!isMember(b1,nexts));
    }
  }
}

void BoardTest::testNextMoves()
{
  {
    Board b(". + . -\n"
	    " . . .\n"
	    ". - + -\n"
	    " + . .\n"
	    ". - . .\n"
	    " . + .\n"
	    ". . . .\n");
    vector<PackedBoard> nexts;
    b.nextMoves(nexts);
    {
      Board b1(". + . -\n"
	       " . . .\n"
	       "+ . + -\n"
	       " . . .\n"
	       ". - . -\n"
	       " . + .\n"
	       ". . . .\n");
      CPPUNIT_ASSERT(isMember(b1,nexts));
    }
    {
      Board b1(". + . -\n"
	       " . . .\n"
	       ". . + -\n"
	       " + . .\n"
	       ". . - .\n"
	       " . - .\n"
	       ". + . .\n");
      CPPUNIT_ASSERT(isMember(b1,nexts));
    }
    {
      Board b1(". + . -\n"
	       " . . .\n"
	       ". . + -\n"
	       " + . .\n"
	       ". - - .\n"
	       " . . .\n"
	       ". + . .\n");
      CPPUNIT_ASSERT(!isMember(b1,nexts));
    }
  }
}

